pub struct RawItem { /* private fields */ }Expand description
A stored entry containing the serialized representation and an optional cached runtime value.
serialized: theserde_value::Valueused for Serialize/Deserialize of the map.value: an optionalBox<dyn Any>holding the deserialized value. This field is not serialized and is cleared onCloneandDeserialize.
Semantics:
- On
insert, bothserializedandvalueare populated. - On access (
get/get_mut), the entry will lazily deserializeserializedintovalueif the cache is empty. into_innerattempts to extract the concrete type from the cache or by deserializing.
Notes:
- The cloning, serialization and deserialization happens based on the serialized
serde_value::Value, so any modifications to fields that are marked#[serde(skip)]will not be lost after a serialization round-trip, or after cloning the map. - Currently any modifications to the cached
valueare not reflected back into theserializedform, so serialization WILL reflect the original value only. This is a known limitation and is planned to be addressed in a future version by returning a Guard object that will update the serialized value on Drop.