SerializableAnyMap

Struct SerializableAnyMap 

pub struct SerializableAnyMap { /* private fields */ }
A serializable heterogeneous-map keyed by the stable type_name::<T>().

SerializableAnyMap behaves like anymap3 for most basic use-cases but stores values in serialized form so the entire map can be serialized and sent across process boundaries. The map implements Serialize and Deserialize where only the serialized form is persisted; the cached deserialized values are not persisted.

Inserted types must implement Serialize + Deserialize<'de> + 'static.

See method docs for usage patterns (insert, get, get_mut, entry, …).

Safety notes:

  • Some escape hatches (as_raw_mut, from_raw) are unsafe — the caller must ensure the key string matches the type stored under it.

A stored entry containing the serialized representation and an optional cached runtime value.

Implementations

Source

impl SerializableAnyMap

pub fn new() -> Self

Create a new empty SerializableAnyMap.

pub fn with_capacity(capacity: usize) -> Self

Create a new SerializableAnyMap with the specified capacity.

pub fn capacity(&self) -> usize

Returns the number of elements the map can hold without reallocating.

pub fn reserve(&mut self, additional: usize)

Reserves capacity for at least additional more elements to be inserted. The collection may reserve more space to avoid frequent reallocations.

Panics

Panics if the new capacity overflows usize.

pub fn shrink_to_fit(&mut self)

Shrinks the capacity of the collection as much as possible. It will drop down as much as possible while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

pub fn len(&self) -> usize

Returns the number of items in the collection.

pub fn is_empty(&self) -> bool

Returns true if there are no items in the collection.

pub fn clear(&mut self)

Removes all items from the collection. Keeps the allocated memory for reuse.

pub fn try_get<T>(&self) -> Option<&T>
where T: for<'de> Deserialize<'de> + Any + 'static,

Get an immutable reference to a cached value of type T if it exists AND was deserialized.

Notes

This does NOT lazily deserialize, use Self::get if you want lazy deserialization. This may return None even if the type is present in the map, but not deserialized.

pub fn get<T>(&mut self) -> Option<Result<&T, DeserializerError>>
where T: Serialize + for<'de> Deserialize<'de> + 'static,

Get an immutable reference to a value of type T, lazily deserializing if necessary.

Notes

This requires a &mut self since it may need to deserialize and modify the entry.

pub fn get_deserialized_copy<T>(&self) -> Option<T>
where T: for<'de> Deserialize<'de> + Any + 'static,

Gets a copy value of type T, by deserializing the value in the map.

Warning

This always runs deserialization, and may return an instance that is not equivalent to the one in the map if any fields are marked #[serde(skip)] for example.

pub fn get_mut<T>( &mut self, ) -> Option<Result<WriteGuard<'_, T>, DeserializerError>>
where T: Serialize + for<'de> Deserialize<'de> + Any + 'static,

Get a mutable reference to a value of type T, lazily deserializing into the cache if necessary.

pub fn get_serialized_value<T>(&self) -> Option<&Value>
where T: 'static,

Get the serialized serde_value::Value for type T if present.

pub unsafe fn insert_value_by_name( &mut self, type_name: String, value: Value, ) -> Option<Value>

Insert by type name key.

This is lower-level and useful if you already have a Value.

Safety

The provided Value needs to match the given matches the type name.

pub fn insert<T>(&mut self, value: T) -> Option<Result<T, DeserializerError>>
where T: Serialize + for<'de> Deserialize<'de> + Any + 'static,

Insert a value of type T. Returns the previous value of that type if present.

pub fn try_insert<'a, T>( &'a mut self, value: T, ) -> Result<WriteGuard<'a, T>, OccupiedError<'a, T>>
where T: Serialize + for<'de> Deserialize<'de> + Any + 'static,

Tries to insert a value into the map, and returns a mutable reference to the value if successful.

If the map already had this type of value present, nothing is updated, and an error containing the occupied entry and the value is returned.

pub fn remove<T>(&mut self) -> Option<T>
where T: Serialize + for<'de> Deserialize<'de> + Any + 'static,

Remove the stored value of type T, returning it if was present, or None if it was not.

pub fn contains<T>(&self) -> bool
where T: 'static,

Returns true if a value of type T exists in the map (regardless whether already deserialized or not).

pub fn contains_deserialized<T>(&self) -> bool
where T: 'static,

Returns true if a value of type T exists in the map and has already been deserialized

pub fn entry<T>(&mut self) -> Entry<'_, T>
where T: Serialize + for<'de> Deserialize<'de> + Any + 'static,

Entry API similar to HashMap::entry / anymap3::entry::<T>().

pub fn keys(&self) -> impl Iterator<Item = &String>

Return an iterator over type-name keys.

pub fn as_raw(&self) -> &HashMap<String, Wrapper>

Get access to the raw hash map that backs this.

This will seldom be useful, but it’s conceivable that you could wish to iterate over all the items in the collection, and this lets you do that.

pub unsafe fn as_raw_mut(&mut self) -> &mut HashMap<String, Wrapper>

Get mutable access to the raw hash map that backs this.

This will seldom be useful, but it’s conceivable that you could wish to iterate over all the items in the collection mutably, or drain or something, or possibly even batch insert, and this lets you do that.

Safety

If you insert any values to the raw map, the key (a String) must match the value’s type name as returned by any::type_name(), or undefined behaviour will occur when you access those values.

(Removing entries is perfectly safe.)

pub fn into_raw(self) -> HashMap<String, Wrapper>

Convert this into the raw hash map that backs this.

This will seldom be useful, but it’s conceivable that you could wish to consume all the items in the collection and do something with some or all of them, and this lets you do that, without the unsafe that .as_raw_mut().drain() would require.

pub unsafe fn from_raw(raw: HashMap<String, Wrapper>) -> SerializableAnyMap

Construct a map from a collection of raw values.

You know what? I can’t immediately think of any legitimate use for this.

Perhaps this will be most practical as unsafe { SerializableAnyMap::from_raw(iter.collect()) }, iter being an iterator over (String, Box<dyn Any + Serialize + Deserialize + 'static>) pairs. Eh, this method provides symmetry with into_raw, so I don’t care if literally no one ever uses it. I’m not even going to write a test for it, it’s so trivial.

Safety

For all entries in the raw map, the key (a String) must match the value’s type as returned by any::type_name(), or undefined behaviour will occur when you access that entry.